@@ -7,9 +7,10 @@ from .cursor import Cursor
 from .pool import create_pool, Pool
 
 
-__all__ = ('connect', 'create_pool', 'Connection', 'Cursor', 'Pool')
+__all__ = ('connect', 'create_pool', 'Connection', 'Cursor', 'Pool',
+           'version', 'version_info')
 
-__version__ = '0.2.3'
+__version__ = '0.3.1'
 
 version = __version__ + ' , Python ' + sys.version
 
