@@ -12,7 +12,14 @@ __all__ = ('connect',)
 
 @asyncio.coroutine
 def connect(dsn=None, *, loop=None, **kwargs):
-    """XXX"""
+    """A factory for connecting to PostgreSQL.
+
+    The coroutine accepts all parameters that psycopg2.connect() does
+    plus optional keyword-only `loop` parameter.
+
+    Returns instantiated Connection object.
+
+    """
     if loop is None:
         loop = asyncio.get_event_loop()
 
@@ -23,7 +30,12 @@ def connect(dsn=None, *, loop=None, **kwargs):
 
 
 class Connection:
-    """XXX"""
+    """Low-level asynchronous interface for wrapped psycopg2 connection.
+
+    The Connection instance encapsulates a database session.
+    Provides support for creating asynchronous cursors.
+
+    """
 
     def __init__(self, dsn, loop, waiter, **kwargs):
         self._loop = loop
@@ -91,7 +103,7 @@ class Connection:
             'connection': self,
             })
         self._close()
-        if self._waiter:
+        if self._waiter and not self._waiter.done():
             self._waiter.set_exception(psycopg2.OperationalError(message))
 
     def _create_waiter(self, func_name):
@@ -116,7 +128,16 @@ class Connection:
     @asyncio.coroutine
     def cursor(self, name=None, cursor_factory=None,
                scrollable=None, withhold=False):
-        """XXX"""
+        """A coroutine that returns a new cursor object using the connection.
+
+        *cursor_factory* argument can be used to create non-standard
+         cursors. The argument must be suclass of
+         `psycopg2.extensions.cursor`.
+
+        *name*, *scrollable* and *withhold* parameters are not supported by
+        psycopg in asynchronous mode.
+
+        """
         impl = yield from self._cursor(name=name,
                                        cursor_factory=cursor_factory,
                                        scrollable=scrollable,
@@ -137,8 +158,8 @@ class Connection:
     @asyncio.coroutine
     def close(self):
         """Remove the connection from the event_loop and close it."""
-        # FIXME: process closing with uncommitted transaction
-        # that should wait for _conn.poll() I guess.
+        # N.B. If connection contains uncommitted transaction the
+        # transaction will be discarded
         self._close()
 
     def _close(self):
@@ -149,22 +170,32 @@ class Connection:
             self._loop.remove_writer(self._fileno)
             self._writing = False
         self._conn.close()
+        if self._waiter is not None and not self._waiter.done():
+            self._waiter.set_exception(
+                psycopg2.OperationalError("Connection closed"))
 
     @property
     def closed(self):
-        """Read-only attribute reporting whether the database connection
-        is open (False) or closed (True)."""
+        """Connection status.
+
+        Read-only attribute reporting whether the database connection is
+        open (False) or closed (True).
+
+        """
         return self._conn.closed
 
+    @property
+    def raw(self):
+        """Underlying psycopg connection object, readonly"""
+        return self._conn
+
     @asyncio.coroutine
     def commit(self):
-        """XXX"""
         raise psycopg2.ProgrammingError(
             "commit cannot be used in asynchronous mode")
 
     @asyncio.coroutine
     def rollback(self):
-        """XXX"""
         raise psycopg2.ProgrammingError(
             "rollback cannot be used in asynchronous mode")
 
@@ -201,6 +232,7 @@ class Connection:
 
     @asyncio.coroutine
     def cancel(self):
+        """Cancel the current database operation."""
         waiter = self._create_waiter('cancel')
         self._conn.cancel()
         yield from self._poll(waiter)
@@ -212,6 +244,12 @@ class Connection:
 
     @property
     def dsn(self):
+        """DSN connection string.
+
+        Read-only attribute representing dsn connection string used
+        for connectint to PostgreSQL server.
+
+        """
         return self._dsn
 
     @asyncio.coroutine
@@ -222,26 +260,35 @@ class Connection:
 
     @property
     def autocommit(self):
-        """XXX"""
+        """Autocommit status"""
         return self._conn.autocommit
 
     @autocommit.setter
     def autocommit(self, val):
-        """XXX"""
+        """Autocommit status"""
         self._conn.autocommit = val
 
     @property
     def isolation_level(self):
-        """XXX"""
+        """Transaction isolation level.
+
+        The only allowed value is ISOLATION_LEVEL_READ_COMMITTED.
+
+        """
         return self._conn.isolation_level
 
     @asyncio.coroutine
     def set_isolation_level(self, val):
+        """Transaction isolation level.
+
+        The only allowed value is ISOLATION_LEVEL_READ_COMMITTED.
+
+        """
         self._conn.set_isolation_level(val)
 
     @property
     def encoding(self):
-        """XXX"""
+        """Client encoding for SQL operations."""
         return self._conn.encoding
 
     @asyncio.coroutine
@@ -250,39 +297,42 @@ class Connection:
 
     @property
     def notices(self):
-        """XXX"""
+        """A list of all db messages sent to the client during the session."""
         return self._conn.notices
 
     @property
     def cursor_factory(self):
-        """XXX"""
+        """The default cursor factory used by .cursor()."""
         return self._conn.cursor_factory
 
     @asyncio.coroutine
     def get_backend_pid(self):
+        """Returns the PID of the backend server process."""
         return self._conn.get_backend_pid()
 
     @asyncio.coroutine
     def get_parameter_status(self, parameter):
+        """Look up a current parameter setting of the server."""
         return self._conn.get_parameter_status(parameter)
 
     @asyncio.coroutine
     def get_transaction_status(self):
+        """Return the current session transaction status as an integer."""
         return self._conn.get_transaction_status()
 
     @property
     def protocol_version(self):
-        """XXX"""
+        """A read-only integer representing protocol being used."""
         return self._conn.protocol_version
 
     @property
     def server_version(self):
-        """XXX"""
+        """A read-only integer representing the backend version."""
         return self._conn.server_version
 
     @property
     def status(self):
-        """XXX"""
+        """A read-only integer representing the status of the connection."""
         return self._conn.status
 
     @asyncio.coroutine
