@@ -89,23 +89,26 @@ class Pool:
                     tran_status)
                 conn._close()
                 return
-            try:
-                self._free.put_nowait(conn)
-            except asyncio.QueueFull:
-                # close the oldest connection in free pool.
-                # that can happen if we acuire multiple
-                # connections from parallel tasks
-                # when minsize == 0
-                conn2 = self._free.get_nowait()
-                conn2._close()
-                assert not self._free.full(), self._free
-                self._free.put_nowait(conn)
+            while True:
+                try:
+                    self._free.put_nowait(conn)
+                except asyncio.QueueFull:
+                    # close the oldest connection in free pool.
+                    # that can happen if we acuire multiple
+                    # connections from parallel tasks
+                    # when minsize == 0
+                    conn2 = self._free.get_nowait()
+                    conn2._close()
+                else:
+                    break
 
     @asyncio.coroutine
-    def cursor(self):
+    def cursor(self, name=None, cursor_factory=None,
+               scrollable=None, withhold=False):
         """XXX"""
         conn = yield from self.acquire()
-        cur = yield from conn.cursor()
+        cur = yield from conn.cursor(name=name, cursor_factory=cursor_factory,
+                                     scrollable=scrollable, withhold=withhold)
         return _CursorContextManager(self, conn, cur)
 
     def __enter__(self):
